#accuracy_scraper.pl
#takes the accuracy out of the report files
#THIS FILE MUST BE EXECUTED INSIDE THE ALL_TESTS DIRECTORY
#usage: perl accuracy_scraper.pl

my @all_dirs;
$all_dirs[0] = "scene_text";
$all_dirs[1] = "scene_text_2";
$all_dirs[2] = "scene_text_3";
my @percent_chars_outside_box;
my @difference_in_rotation_angle;
my @percent_false_positives;
my @percent_false_negatives;
my @rotation_angles;
my $file_count = 0;

foreach $dir (@all_dirs) {
    
    chdir $dir;
    
    my @all_files = glob "*";
    
    foreach $file (@all_files) {
	if ($file =~ m/IMAGE/) {
	    my $is_a_directory = 0;
	    $is_a_directory = 1 if -d $file;
	    
	    unless ($is_a_directory == 1) {
		next;
	    }

	    chdir $file;

	    open IDEAL, "<ideal_results.txt";
	    
	    while (defined($line = <IDEAL>)) {
		if ($line =~ m/percent_characters_missed =/) {
		    my @fields = split(/\s+/, $line);
		    if ($fields[2] =~ m/(-*\d+.*\d*)/) {
			$percent_characters_outside_box[$file_count] = $1;
		    }
		}

		if ($line =~ m/rotation_angle =/) {
		    my @fields = split(/\s+/, $line);
		    if ($fields[2] =~ m/(-*\d+.*\d*)/) {
			$rotation_angles[$file_count] = $1;
		    }
		}

		if ($line =~ m/rotation_diff =/) {
		    my @fields = split(/\s+/, $line);
		    if ($fields[2] =~ m/(-*\d+.*\d*)/) {
			$difference_in_rotation_angle[$file_count] = $1;
		    }
		}

		if ($line =~ m/percent_pos_false =/) {
		    my @fields = split(/\s+/, $line);
		    if ($fields[2] =~ m/(-*\d+.*\d*)/) {
			$percent_false_positives[$file_count] = $1;
		    }
		}

		if ($line =~ m/percent_text_neg =/) {
		    my @fields = split(/\s+/, $line);
		    if ($fields[2] =~ m/(-*\d+.*\d*)/) {
			$percent_false_negatives[$file_count] = $1;
		    }
		}
	    }
	    	    
	    chdir "..";
	    $file_count += 1;
	    close IDEAL;
	}
    }

    chdir "..";
}

#print average results here
print "Average percent characters outside box: " . average(\@percent_characters_outside_box) . "\n";
print "Average error in rotation angle: " . average(\@difference_in_rotation_angle) . "\n";
print "Average percent false positives for text: " . average(\@percent_false_positives) . "\n";
print "Average percent false negatives for text: " . average(\@percent_false_negatives) . "\n";

print "Standard deviation of percent characters outside box: " . standard_dev(\@percent_characters_outside_box) . "\n";
print "Standard deviation of error in rotation angle: " . standard_dev(\@difference_in_rotation_angle) . "\n";
print "Standard deviation of percent false positives for text: " . standard_dev(\@percent_false_positives) . "\n";
print "Standard deviation of percent false negatives for text: " . standard_dev(\@percent_false_negatives) . "\n";

print "\n";
#take absolute value of errors and find average
for (my $index = 0; defined($difference_in_rotation_angle[$index]); $index++) {
    $difference_in_rotation_angle[$index] = abs($difference_in_rotation_angle[$index]);
}

#print average absolute results here
print "Average absolute error in rotation angle: " . average(\@difference_in_rotation_angle) . "\n";

print "Standard deviation of absolute error in rotation angle: " . standard_dev(\@difference_in_rotation_angle) . "\n";


print "\n";

#find maximum and minimum rotation angles
$max = $rotation_angles[0];
$min = $rotation_angles[0];

foreach $i (@rotation_angles[1..$#rotation_angles]) {
    if ($i > $max)
    {
	$max = $i;
    }
    elsif ($i < $min)
    {
	$min = $i;
    }
}

print "Maximum actual angle of rotation: $max\n";
print "Minimum actual angle of rotation: $min\n";

sub average {
    my $pointer = $_[0];

    my $running_sum = 0;
    my $count = 0;
    while (defined($number = $pointer->[$count])) {
	$running_sum += $number;
	$count += 1;
    }
    return $running_sum / $count;
}

sub standard_dev {
    my $pointer = $_[0];

    my $mean1 = average($pointer);
        
    my $running_sum = 0;
    my $count = 0;
    while (defined($number = $pointer->[$count])) {
	$running_sum += ($mean1 - $number)**2;
	$count += 1;
    }
    my $mean2 = $running_sum / $count;

    return sqrt($mean2);
}
